from orchestrator.forms import FormPage
from orchestrator.targets import Target
from orchestrator.types import FormGenerator, State, UUIDstr
from orchestrator.workflow import StepList, done, init, step, workflow
from orchestrator.workflows.steps import resync, store_process_subscription, unsync
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products.product_types.iptrunk import Iptrunk
from gso.services import provisioning_proxy
from gso.services.provisioning_proxy import pp_interaction


def initial_input_form_generator(subscription_id: UUIDstr) -> FormGenerator:
    subscription = Iptrunk.from_subscription(subscription_id)

    class ModifyIptrunkForm(FormPage):
        tt_number: str
        isis_metric: int = subscription.iptrunk.iptrunk_isis_metric

    user_input = yield ModifyIptrunkForm

    return user_input.dict()


@step("Update subscription")
def modify_iptrunk_subscription(subscription: Iptrunk, isis_metric: int) -> State:
    subscription.iptrunk.iptrunk_isis_metric = isis_metric

    return {"subscription": subscription}


@step("Provision IP trunk ISIS interface [DRY RUN]")
def provision_ip_trunk_isis_iface_dry(
    subscription: Iptrunk, process_id: UUIDstr, callback_route: str, tt_number: str
) -> State:
    provisioning_proxy.provision_ip_trunk(subscription, process_id, callback_route, tt_number, "isis_interface")

    return {"subscription": subscription}


@step("Provision IP trunk ISIS interface [FOR REAL]")
def provision_ip_trunk_isis_iface_real(
    subscription: Iptrunk, process_id: UUIDstr, callback_route: str, tt_number: str
) -> State:
    provisioning_proxy.provision_ip_trunk(subscription, process_id, callback_route, tt_number, "isis_interface", False)

    return {"subscription": subscription}


@workflow(
    "Modify IP trunk",
    initial_input_form=wrap_modify_initial_input_form(initial_input_form_generator),
    target=Target.MODIFY,
)
def modify_isis_metric() -> StepList:
    return (
        init
        >> store_process_subscription(Target.MODIFY)
        >> unsync
        >> modify_iptrunk_subscription
        >> pp_interaction(provision_ip_trunk_isis_iface_dry)
        >> pp_interaction(provision_ip_trunk_isis_iface_real)
        >> resync
        >> done
    )
